/*                               -*- Mode: C -*- 
 * tuple-list.c --- This program contains utility functions 
 * Author          : Javed Chowdhury.
 * Created On      : 
 */

#include "tuple-list.h" 

extern list *TupleList_temp; /* externaly defined list to hold the 
                                result of one parse */
extern list *AttrList_temp;

tuple factAttribute(tuple Node,list *aList)
/****************************************************************
 Adds a Node in the temporary attributes list.
 */
{
  addElement(aList, copyTuple(Node));
 
  return(Node);
}



tuple makeTuple(char *relation, char *firstId, char *secondId)
/**************************************************
 * generates and returns a tuple with the given fields
 */
{
  tuple fact;

  fact = initTuple();
  TPIDENT(fact) = dupString(firstId);
  
  if (strcmp(relation, "$INSTANCE") == 0) {
    TPTYPE(fact) = _instance;
    INSTARG2(fact) = str2Instance( secondId);
  }
  else {
    TPTYPE(fact) = _relation;
    REL_REL(fact) = str2TalRel(relation);
    RELARG2(fact) = dupString(secondId);
  }
  
  return(fact);
}





tuple factTuple(char *relation, char *firstId, char *secondId)
/****************************************************************
 Creates a factuple and add it to the temporary tuple list.
 */
{
  tuple fact;

  fact = makeTuple(relation, firstId, secondId);
  addElement(TupleList_temp, fact);

  return(fact);
}

/************************************************************
This function seems obsolete to me.... But
void ifListToMainList(list *IFlist)


{
  tuple fact;

  if (! preprocessing) {  
	while (emptyList(IFlist) == FALSE) {
	  fact = delElement(IFlist);

	  switch (TPTYPE(fact)) {
	  case _instance:
		printf("$INSTANCE %s %s\n", INSTARG1(fact),
			                        instance2str(INSTARG2(fact)));
		free (fact);
		break;

	  case _relation:
		printf("%s %s %s\n", talRel2str(REL_REL(fact)),
                             RELARG1(fact), RELARG2(fact));
		free (fact);
		break;

	  case _attribute:
		addElement(mainList, fact);

	  default:
		break;
	  }
	}
  }
}
 */

void printFListTuple(FILE *fout, list *l)
/**************************************************
 * Prints a list of tuples l to the file fout
 */
{
  listElt *lelt;

  if (emptyList(l)) return;
  lelt = l->first;
  while (lelt != NULL) {
	printFTuple(fout,lelt->item);
	lelt = lelt->next;
  }
	
}


void destroy_Tlist(list *l)
/***************************************************
 * free all tuples of list l
 */
{
  if (l != NULL) {
	while (! emptyList(l))
	  delTuple(delElement(l));
	free(l);
  }
}


bool tuple_included(tuple t, list *l)
/***************************************************
 * returns TRUE if t is included in l
 */
{
  listElt *lelt;

  if (emptyList(l)) return(FALSE);
  lelt = l->first;
  while (lelt != NULL) {
	if (cmpTuple(t,lelt->item) == 0) return(TRUE);
	lelt = lelt->next;
  }
  return(FALSE);
}

void insert_elements(list *new, list *old)
/**************************************************
 * Insert the elements of list new in old (when non redundants).
 * old and new are non empty lists.
 */
{
  listElt *new_elt;

  new_elt = new->first;
  while (new_elt != NULL) {
	if (!tuple_included(new_elt->item, old)) 
	  addElement(old, copyTuple(new_elt->item));
	new_elt = new_elt->next;
  }
}

void merge_tuple_lists(list *new, list *old)
/**************************************************
 * Add all the elements of the list new to old in 
 * ascending order according to the list elements identifier
 */
{
  if (emptyList(old) == TRUE) {
	old->first = new->first;
	old->last = new->last;
	new->first = NULL;
	new->last = NULL;
	free(new); 
  }
  else {
	/* insert all the non redundants elements of new into old */
	insert_elements(new,old);
	destroy_Tlist(new);
  }
}


/**************************************************
 * Local Variables: 
 * tab-width: 4
 * End:
 */

